/*****************************************************************************
 * example2.cpp
 ***************************************************************************** 
 *
 * Projekt:     Halcon/C++
 * Author:      W. Eckstein
 * Description: Example for Halcon/C++
 *
 * (c) 1996-98 by MVTec Software GmbH
 *                Orleansstr. 34, D-81667 Muenchen, Germany
 *****************************************************************************
 *  Procedures: Copy()
 *                - read image
 *                - copy image
 *                - invert new image 2 times
 *              Bit()
 *                - Left- and right shift of the grayvalues 
 *                - Cutting out bit-slices
 *              Morphology()
 *                - Segmentation with threshold
 *                - Dilatation with circle radius 2 in a loop
 *              RegionOfInterest()
 *                - Restriction of the definition range
 *                - Output of image sections
 *****************************************************************************
 *
 * $Revision: 1.2 $
 * $Date: 1998/02/24 10:44:06 $
 */




#include "HalconCpp.h"
#include <iostream.h>
 

void Copy()
{
  double t1,t2;

  HByteImage in("mreut");
  HWindow    w;

  in.Display(w);

  HByteImage out = in;

  int  width  = out.Width();
  int  height = out.Height();

  long end    = width * height;

  (void)::count_seconds(&t1);

  for (int k = 0; k < end; k++)
  {
    out[k] = 255 - in[k];
  }

  (void)::count_seconds(&t2);

  cout << "(linear) time = " << t2-t1 << " sec" << endl;

  cout << "Transformed !" << endl;  out.Display(w);   w.Click();
  cout << "Original !"    << endl;  in.Display(w);    w.Click();

  (void)::count_seconds(&t1);

  for (int y=0; y<height; y++)
  {
    for (int x=0; x<width; x++)
    {
      out(x,y) = 255 - out(x,y);
    }
  }

  (void)::count_seconds(&t2);

  cout << "(x,y) time = " << t2-t1 << " sec" << endl;
  cout << "back transformed" << endl;

  out.Display(w);
}


void Bit()
{
  HByteImage image("mreut");
  HWindow  w;
  image.Display(w);

  int i;
  // shift right (make image darker)
  for (i=1; i<8; i++)
  {
    (image >> i).Display(w);
    printf(">> %d\n",i);
  }
  // shift left (make image brighter)
  for (i=1; i<8; i++)
  {
    (image << i).Display(w);
    printf("<< %d\n",i);
  }
  // cutting out of a bit slice
  for (i=0; i<8; i++)
  {
    HRegion reg = (image & (1<<i)) != 0;
    w.ClearWindow();
    reg.Display(w);
    printf("& Bit %d\n",i);
  }
  // image brighter
  printf("End of shift left\n");
  (image * 1.5).Display(w);
  printf("Image brighter\n");
}


void Morphology()
{
  HByteImage g("monkey");
  HWindow w;
  HRegion reg1 = g >= 128;
  HRegion reg2 = reg1;
  w.SetColor("white");
  w.SetInsert("xor");
  reg1.Display(w);
  for (int i=0; i<100; i++)
  {
    (reg1++).Display(w);
  }
  (void)w.Click();
  w.ClearWindow();
}


void RegionOfInterest()
{
  HByteImage image("mreut");
  HWindow    w;
  int        i;
  for (i=1; i<30; i += 1)
    (image & HRegion::GenCircle(255,255,i)).Display(w);
  for (; i<80; i += 2)
    (image & HRegion::GenCircle(255,255,i)).Display(w);
  for (; i<255; i += 4)
    (image & HRegion::GenCircle(255,255,i)).Display(w);
  (void)w.Click();
}



main()
{
  Bit();
  Copy();
  Morphology();
  RegionOfInterest();
  return(0);
}


