/*****************************************************************************
 * example8.cpp
 ***************************************************************************** 
 *
 * Projekt:     Halcon/C++
 * Author:      W. Eckstein
 * Description: Example for Halcon/C++
 *
 * (c) 1996-98 by MVTec Software GmbH
 *                Orleansstr. 34, D-81667 Muenchen, Germany
 *****************************************************************************
 * Two operations for testing the tuple mode    
 * 1. Informations about prozedures  (proc_name, proc_info1)
 *    First the names of all Halcon procedures are queried 
 *    With help of these names access for the name of the parameters 
 *    Execution in:  fetch_proc_info  
 * 2. Informations about the system status of Halcon (get_system) 
 *    First the names of all system parameters are queried 
 *    Appropriate the current value of every parameter
 *    Execution in:  fetch_sys_info  
 *****************************************************************************
 *
 * $Revision: 1.2 $
 * $Date: 1998/02/24 10:44:06 $
 */



#include "HalconCpp.h"
#include <iostream.h>
#include <stdio.h>


// help procedure to output the parameter of a Halcon procedure
void print_parameter(const HTuple &tupel, const char *art)
{
  int   i,n;
  
  n = tupel.Num();           // number of parameters in one class
  if (n > 0)                 // parameter available ?
  {
    (void)printf("  %s => ",art);
    for (i=0; i<n; i++)      // output all
      (void)printf("%s ",tupel[i].S());
    (void)printf("\n");
  }
}




// Informatios about procedures: name + parameters
void fetch_proc_info()
{
  HTuple  Query;
  HTuple  P1,P2,P3,P4;
  int     i,num;
  
  // Query: The name of all Halcon procedures 
  (void)::get_operator_name("",&Query); // get all names
  
  num = Query.Num();  
  (void)printf("Total number of procedures: %d\n",num);
  for (i=0; i<num; i++)
  {
    (void)printf("Proc #%d: %s\n",i,Query[i].S());   // current proc-name
    
    // Subquery for a procedure:  get 4 parameter classes
    (void)::get_param_names(Query[i].S(),&P1,&P2,&P3,&P4);
    print_parameter(P1,"Input objekt parameter");
    print_parameter(P2,"Output objekt parameter");
    print_parameter(P3,"Input control parameter");
    print_parameter(P4,"Output control parameter");
  }
}



// General system informations about Halcon
void fetch_sys_info()
{
  HTuple  SysFlags,Info;
  int     i,num;
  
  (void)printf("System informations\n");
  (void)::get_system("?",&SysFlags); // Query of HORUS
  num = SysFlags.Num();  // Number of system parameter names
  for (i=0; i<num; i++)  // Get values of all parameters 
  {
    (void)printf("%s ",SysFlags[i].S()); // output names 
    (void)::get_system(SysFlags[i].S(),&Info); 
    if (Info.Num() > 0)
    {
      switch (Info[0].ValType()) {       // Which type has system parameter ?
        case LONG_PAR:
	  (void)printf("(long)  %ld\n",Info[0].L());  // Value output
	  break;
        case DOUBLE_PAR:
	  (void)printf("(double) %lf\n",Info[0].D());
	  break;
        case STRING_PAR:
	  (void)printf("(string) %s\n",Info[0].S());
	  break;
      }
    }
    else
      (void)printf("(--) no data\n");
  }
}

 




main()
{
  fetch_sys_info();
  fetch_proc_info();

  return(0);
}


